import { createSignal, createResource, For } from "solid-js";


export default function SolidPersonTableServer(props) {
  let baseUrl = 'http://localhost:3000/api/';
  let baseUrlImg = 'http://localhost:3000/img/';

  const [filterString, setFilterString] = createSignal("");
  const [name, setName] = createSignal("");
  const [license, setLicense] = createSignal("");
  const [id, setId] = createSignal("");
  const [changeCarModalOpen, setChangeCarModalOpen] = createSignal(false);
  const [newCarModalOpen, setNewCarModalOpen] = createSignal(false);

  const fetchCarsRessource = async() => {
    let data = await fetch(baseUrl + 'car');
    let json = await data.json(); 
    return json.carsObject.cars;
  } 
  const [cars, { refetch: carsRefetch }] = createResource(fetchCarsRessource); 

  const postCar = async() => {
        const requestOptions = {
            method: 'POST',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ name: name(), license: license() })
        };
        let data = await fetch(baseUrl+'car', requestOptions);
        let json = await data.json(); 
        console.log(json);
        carsRefetch();
   }

   const putCar = async() => {
   const requestOptions = {
            method: 'PUT',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ id: id(), name: name(), license: license()})
        };
        let data = await fetch(baseUrl+'car', requestOptions);
        let json = await data.json(); 
        carsRefetch();
   }

   const deleteCar = async(id) => {
        const requestOptions = {
            method: 'DELETE',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ id: id })
        };
        let data = await fetch(baseUrl+'car', requestOptions);
        let json = await data.json(); 
        carsRefetch();
   }
//           <div><img src={ baseUrlImg + "zen.jpg" } alt="Zen" class="h-auto max-w-full rounded-full" /></div>


   return (
        <div class="text">
           <h1 class="text-3xl my-5">Autos</h1>
           <button onClick = { () => { setId(""); setName(""); setLicense("");  setNewCarModalOpen(true); }} class="my-10 bg-transparent hover:bg-blue-500 text-blue-700 font-semibold hover:text-white py-2 px-4 border border-blue-500 hover:border-transparent rounded">
               Neues Fahrzeug anlegen                    
           </button>
           <div>
            <label for="filter" class="block mb-2 text-sm font-medium text-gray-900 dark:text-white">Filtern</label>
            <input id="filter" type="text" onInput = { (e) => setFilterString(e.target.value) } placeholder="Filtern..." class="input input-bordered w-full max-w-xs" />
           </div>
            <div class="my-5">
                {  cars.loading ? "loading..." :  ""}
           </div>
           <table class="table table-zebra w-full">
                <thead>
                    <tr class="px-6 py-3">
                        <th class="text-lg">Name</th>
                        <th class="text-lg">Kennzeichen</th>
                        <th class="text-lg">Löschen</th>
                        <th class="text-lg">Ändern</th>
                    </tr>
                </thead>
                <tbody>    
                    { cars() && cars().filter( (car) => car.name.toLowerCase().indexOf(filterString().toLowerCase()) >= 0).map( car => 
                        <tr> 
                            <th scope="row">
                                { car.name }
                            </th>
                            <td for="change-car-modal">
                                { car.license }
                            </td>
                            <td class=" py-4">
                                <button onClick = {() => deleteCar(car.id)} class="bg-transparent hover:bg-red-500 text-red-700 font-semibold hover:text-white py-2 px-4 border border-red-500 hover:border-transparent rounded">
                                    Löschen
                                </button>
                            </td>
                            <td class=" py-4">
                                <button onClick = { () => { setId(car.id); setName(car.name); setLicense(car.license); setChangeCarModalOpen(true); }} class="bg-transparent hover:bg-red-500 text-red-700 font-semibold hover:text-white py-2 px-4 border border-red-500 hover:border-transparent rounded">
                                    Ändern
                                </button>
                            </td>
                        </tr> )
                    }
                </tbody>
           </table>
           
           <div class="py-10">
            	
            {/* Modal zum Neuanlegen des Fahrzeugs */}
                <div class={newCarModalOpen() ? "modal modal-open" : "modal"}>
                    <div class="modal-box">
                        <h3 class="font-bold text-lg">Neues Fahrzeug anlegen</h3>
                        <div class="flex flex-wrap -mx-3 mb-6">
                                <div class="w-full md:w-1/2 px-3 mb-6 md:mb-0 py-3">
                                    <label class="block uppercase tracking-wide text-gray-700 text-m font-bold mb-2" for="car-name">
                                        Name
                                    </label>
                                    <input value={name()} onInput = { (e) => setName(e.target.value) }  class="appearance-none block w-full bg-gray-200 text-gray-700 border border-red-500 rounded py-3 px-4 mb-3 leading-tight focus:outline-none focus:bg-white" id="car-name" type="text" placeholder="Name"/>
                                </div>
                                <div class="w-full md:w-1/2 px-3  py-3">
                                    <label class="block uppercase tracking-wide text-gray-700 text-m font-bold mb-2" for="grid-license">
                                        Kennzeichen
                                    </label>
                                    <input value={license()} onInput = { (e) => setLicense(e.target.value) }  class="appearance-none block w-full bg-gray-200 text-gray-700 border border-gray-200 rounded py-3 px-4 leading-tight focus:outline-none focus:bg-white focus:border-gray-500" id="grid-license" type="text" placeholder="Kennzeichen"/>
                                </div>
                        </div>
                        <div class="modal-action">
                            <label for="modal-new-car" class="btn" onClick={ () => { postCar(); setNewCarModalOpen(false); } }>Speichern</label>
                            <label for="modal-new-car" class="btn" onClick={ () => setNewCarModalOpen(false)}>Abbrechen</label>
                        </div>
                    </div>
                    
                </div>
            </div>   
            <div> 
                {/* Modal zum Ändern des Fahrzeugs */}
                <div class={changeCarModalOpen() ? "modal modal-open" : "modal"}>
                    <div class="modal-box">
                        <h3 class="font-bold text-lg">Fahrzeugdaten ändern</h3>
                        <div class="flex flex-wrap -mx-3 mb-6 py-6">
                            <div class="w-full md:w-1/2 px-3 mb-6 md:mb-0">
                            <label class="block uppercase tracking-wide text-gray-700 text-m font-bold mb-2" for="grid-first-name">
                                Name
                            </label>
                                <input value={name()} onInput = { (e) => setName(e.target.value) }  class="appearance-none block w-full bg-gray-200 text-gray-700 border border-red-500 rounded py-3 px-4 mb-3 leading-tight focus:outline-none focus:bg-white" id="grid-first-name" type="text" placeholder="Name"/>
                            </div>
                            <div class="w-full md:w-1/2 px-3">
                            <label class="block uppercase tracking-wide text-gray-700 text-m font-bold mb-2" for="grid-last-name">
                                Kennzeichen
                            </label>
                            <input value={license()} onInput = { (e) => setLicense(e.target.value) }  class="appearance-none block w-full bg-gray-200 text-gray-700 border border-gray-200 rounded py-3 px-4 leading-tight focus:outline-none focus:bg-white focus:border-gray-500" id="grid-last-name" type="text" placeholder="Kennzeichen"/>
                        </div>
                        <div class="modal-action">
                            <label for="my-modal" class="btn" onClick={ () => { putCar(); setChangeCarModalOpen(false); } }>Speichern</label>
                            <label for="my-modal" class="btn" onClick={ () => setChangeCarModalOpen(false)}>Abbrechen</label>
                        </div>
                    </div>
                    </div>
                </div>
            </div>
    </div>)  
}
