import { createSignal, createResource } from "solid-js";
import {createDropzone} from "@solid-primitives/upload"

export default function SolidPersonTableServer(props) {
  let baseUrl = 'http://localhost:3000/api/person'
  let baseUrlImg = 'http://localhost:3000/api/imgupload'
  let baseUrlPersonImg = 'http://localhost:3000/api/imgperson'
  let imgUrl = 'http://localhost:3000/img/'
  const [filterString, setFilterString] = createSignal("");
  const [prename, setPrename] = createSignal("");
  const [surname, setSurname] = createSignal("");
  const [age, setAge] = createSignal("");
  const [city, setCity] = createSignal("");
  const [id, setId] = createSignal("");
  const [imgFile, setImgFile] = createSignal({});
 
   const fetchPersonsRessource = async() => {
     let data = await fetch(baseUrl);
     let json = await data.json(); 
     console.log("data fetched", json)
     return json.personObject.persons;
   } 
   const [persons, { refetch }] = createResource(fetchPersonsRessource); 

   const postPerson = async() => {
        const requestOptions = {
            method: 'POST',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ prename: prename(), surname: surname(), age: age(), city: city()})
        };
        let data = await fetch(baseUrl, requestOptions);
        let json = await data.json(); 
        console.log(json);
        refetch();
   }

   const putPerson = async() => {
    const requestOptions = {
        method: 'PUT',
        headers: { 'Content-Type': 'application/json' },
        body: JSON.stringify({ id: id(), prename: prename(), surname: surname(), age: age(), city: city()})
    };
    let data = await fetch(baseUrl, requestOptions);
    let json = await data.json(); 
    console.log(json);
    refetch();
   }

   const deletePerson = async(id) => {
    const requestOptions = {
        method: 'DELETE',
        headers: { 'Content-Type': 'application/json' },
        body: JSON.stringify({ id: id})
    };
    let data = await fetch(baseUrl, requestOptions);
    let json = await data.json(); 
    refetch();
   }

   const uploadImage = async(e) => {
        console.log("e")
        console.log(e)
        console.log("file.target.files[0]")
        console.log(e.target.files[0])
        const data = new FormData();
        data.append("image", e.target.files[0]);
        data.append("type", e.target.files[0].type);
        data.append("name", e.target.files[0].name);
        const requestOptions = {
            method: 'POST',
            body: data
        };
        let response = await fetch(baseUrlImg, requestOptions);
        let json = await response.json(); 
   }
  
   const uploadPersonImage = async(personId, file) => {
        console.log("upload person image")
        console.log(file)
        console.log("personId")
        console.log(personId)        
        const data = new FormData();
        data.append("image", file);
        data.append("type", file.type);
        data.append("name", file.name);
        data.append("personId", personId);
        const requestOptions = {
            method: 'POST',
            body: data
        };
        let response = await fetch(baseUrlPersonImg, requestOptions);
        let json = await response.json(); 
        refetch();
   }

   return (
    <div>
        <div class="text">
           <h1 class="text-3xl my-5">Personen</h1>
           <div>
            <input onInput = { (e) => uploadImage(e) }  type="file" class="file-input w-full max-w-xs" />
           </div>
           <div>
            <label for="filter" class="block mb-2 text-sm font-medium text-gray-900 dark:text-white">Filtern</label>
            <input onInput = { (e) => setFilterString(e.target.value) }   type="text" id="filter" class="bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500" placeholder="Filter..." required></input>
           </div>
           <div class="my-5">
                {  persons.loading ? "loading..." :  ""}
           </div>
           <table class="table table-zebra w-full">
                <thead>
                    <tr class="px-6 py-3">
                        <th>Prename</th>
                        <th>Surname</th>
                        <th>Age</th>
                        <th>City</th>
                        <th>Upload</th>
                        <th>Img</th>
                        <th>Löschen</th>
                    </tr>
                </thead>
                <tbody>    
                    { persons() && persons().filter( (person) => person.prename.toLowerCase().indexOf(filterString().toLowerCase()) >= 0 || person.surname.toLowerCase().indexOf(filterString().toLowerCase()) >= 0).map( (person) => 
                        <tr onClick={ () => { setId(person.id); setPrename(person.prename); setSurname(person.surname); setAge(person.age); setCity(person.city); }}> 
                            <th scope="row" class=" py-4 font-medium text-gray-900 whitespace-nowrap dark:text-white">
                                { person.prename }
                            </th>
                            <td class=" py-4">
                                { person.surname }
                            </td>
                            <td class=" py-4">
                                { person.age }
                            </td>
                            <td class=" py-4">
                                { person.city }
                            </td>
                            <td class=" py-4">
                                <input onInput = { (e) => uploadPersonImage(person.id, e.target.files[0]) }  type="file" class="file-input w-full max-w-xs" />
                            </td>
                            <td class=" py-4">
                                <div class="w-20 rounded">
                                    { person.img !== "" ? <img src={  imgUrl + person.img } alt="Persons" /> : "" }
                                </div>
                            </td>
                            <td class=" py-4">
                            <button onClick = {() => deletePerson(person.id)} class="btn btn-error">
                                Löschen
                            </button>
                            </td>
                        </tr> )
                    }
                </tbody>
           </table>
           <div class="py-10">
            <form class="w-full max-w-lg">
                    <div class="flex flex-wrap -mx-3 mb-6">
                        <div class="w-full md:w-1/2 px-3 mb-6 md:mb-0">
                        <label class="block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2" for="grid-first-name">
                            Vorname
                        </label>
                        <input value={prename()} onInput = { (e) => setPrename(e.target.value) }  class="appearance-none block w-full bg-gray-200 text-gray-700 border border-red-500 rounded py-3 px-4 mb-3 leading-tight focus:outline-none focus:bg-white" id="grid-first-name" type="text" placeholder="Vorname"/>
                        {/* <p class="text-red-500 text-xs italic">Please fill out this field.</p> */}
                        </div>
                        <div class="w-full md:w-1/2 px-3">
                        <label class="block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2" for="grid-last-name">
                            Nachname
                        </label>
                        <input value={surname()} onInput = { (e) => setSurname(e.target.value) }  class="appearance-none block w-full bg-gray-200 text-gray-700 border border-gray-200 rounded py-3 px-4 leading-tight focus:outline-none focus:bg-white focus:border-gray-500" id="grid-last-name" type="text" placeholder="Nachname"/>
                        </div>
                    </div>
                    <div class="flex flex-wrap -mx-3 mb-6">
                        <div class="w-full md:w-1/2 px-3 mb-6 md:mb-0">
                        <label class="block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2" for="grid-city">
                            Stadt
                        </label>
                        <input value={city()} onInput = { (e) => setCity(e.target.value) } class="appearance-none block w-full bg-gray-200 text-gray-700 border border-gray-200 rounded py-3 px-4 leading-tight focus:outline-none focus:bg-white focus:border-gray-500" id="grid-city" type="text" placeholder="Stadt eingeben..."/>
                        </div>
                        <div class="w-full md:w-1/2 px-3 mb-6 md:mb-0">
                        <label class="block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2" for="grid-zip">
                            Alter
                        </label>
                        <input value={age()} onInput = { (e) => setAge(e.target.value) } class="appearance-none block w-full bg-gray-200 text-gray-700 border border-gray-200 rounded py-3 px-4 leading-tight focus:outline-none focus:bg-white focus:border-gray-500" id="grid-zip" type="text" placeholder="Alter eingeben"/>
                        </div>
                    </div>
                    <div class="flex flex-wrap -mx-3 mb-2 px-3 py-3">
                         <button class="bg-blue-500 hover:bg-blue-700 text-white font-bold py-2 px-4 rounded-full" onClick={() => postPerson() }>Speichern</button>
                         <button class="bg-blue-500 hover:bg-blue-700 text-white font-bold mx-6 py-2 px-4 rounded-full" onClick={() => putPerson() }>Ändern</button>
                    </div>
                </form>
            </div>
           </div>
    </div>
    )  
}
