import { createSignal, createResource, For } from "solid-js";
import { Dynamic } from "solid-js/web";
import dayjs from "dayjs";

export default function SolidPersonTableServer(props) {
  let baseUrl = 'http://localhost:3000/api/';

  const [filterString, setFilterString] = createSignal("");
  const [name, setName] = createSignal("");
  const [license, setLicense] = createSignal("");
  const [id, setId] = createSignal("");
  const [start, setStart] = createSignal("");
  const [end, setEnd] = createSignal("");
  const [person, setPerson] = createSignal({}); 
  const [car, setCar] = createSignal({}); 
  const [changeRentalModalOpen, setChangeRentalModalOpen] = createSignal(false);
  const [newRentalModalOpen, setNewRentalModalOpen] = createSignal(false);
  const [newPerson, setNewPerson] = createSignal({}); 
  const [newCar, setNewCar] = createSignal({}); 
  const [newStart, setNewStart] = createSignal("");
  const [newEnd, setNewEnd] = createSignal("");
  

  const fetchPersonsRessource = async() => {
    let data = await fetch(baseUrl + 'person');
    let json = await data.json(); 
    return json.personObject.persons;
  } 
  const [persons] = createResource(fetchPersonsRessource); 

  const fetchCarsRessource = async() => {
    let data = await fetch(baseUrl + 'car');
    let json = await data.json(); 
    return json.carsObject.cars;
  } 
  const [cars] = createResource(fetchCarsRessource); 

  const fetchRentalsRessource = async() => {
    let data = await fetch(baseUrl + 'rental');
    let json = await data.json(); 
    console.log(json)
    return json.rentalsObject.rentals;
  } 
  const [rentals, { refetch }] = createResource(fetchRentalsRessource); 


  const postRental = async() => {
        const requestOptions = {
            method: 'POST',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ carId: newCar().id, personId: newPerson().id, start: newStart(), end: newEnd() })
        };
        let data = await fetch(baseUrl+'rental', requestOptions);
        let json = await data.json(); 
        refetch();
   }

   const putRental = async() => {
        const requestOptions = {
            method: 'PUT',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ id: id(), carId: car().id, personId: person().id, start: start(), end: end()})
        };
        let data = await fetch(baseUrl+'rental', requestOptions);
        refetch();
   }


   const deleteRental = async(id) => {
        const requestOptions = {
            method: 'DELETE',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ id: id})
        };
        let data = await fetch(baseUrl+'rental', requestOptions);
        let json = await data.json(); 
        refetch();
   }

   return (
        <div class="text">
           <h1 class="text-3xl">Ausleihen</h1>
           <div class="my-5">
                {  rentals.loading ? "loading..." :  ""}
           </div>
           <button onClick = { () => { setId(""); setPerson({}); setCar({}); setStart(""); setEnd(""); setNewRentalModalOpen(true); }} class="my-10 bg-transparent hover:bg-blue-500 text-blue-700 font-semibold hover:text-white py-2 px-4 border border-blue-500 hover:border-transparent rounded">
               Neue Auleihe                    
           </button>
           <table class="table table-zebra w-full">
                <thead>
                    <tr class="px-6 py-3">
                        <th>Person</th>
                        <th>Auto</th>
                        <th>Start</th>
                        <th>Ende</th>
                        <th>Löschen</th>
                        <th>Ändern</th>
                    </tr>
                </thead>
                <tbody>    
                    { rentals() && rentals().map( rental => 
                        <tr> 
                            <td>
                                {persons() && persons().find(person => person.id === rental.personId) !== undefined ? persons().find(person => person.id === rental.personId).prename + " " + persons().find(person => person.id === rental.personId).surname : "- ungültig -"}
                            </td>
                            <td>
                                {cars() && cars().find(car => car.id === rental.carId) !== undefined ? cars().find(car => car.id === rental.carId).name + " | " + cars().find(car => car.id === rental.carId).license : "- ungültig -"}
                            </td>
                            <td>
                                { rental.start }
                            </td>
                            <td>
                                { rental.end }
                            </td>
                            <td class=" py-4">
                                <button onClick = {() => deleteRental(rental.id)} class="bg-transparent hover:bg-red-500 text-red-700 font-semibold hover:text-white py-2 px-4 border border-red-500 hover:border-transparent rounded">
                                    Löschen
                                </button>
                            </td>
                            <td class=" py-4">
                                <button onClick = { () => { setId(rental.id); setPerson(persons().find(person => person.id === rental.personId)); setCar(cars().find(car => car.id === rental.carId)); setStart(rental.start); setEnd(rental.end); setChangeRentalModalOpen(true); }} class="bg-transparent hover:bg-red-500 text-red-700 font-semibold hover:text-white py-2 px-4 border border-red-500 hover:border-transparent rounded">
                                    Ändern
                                </button>
                            </td>
                        </tr> )
                    }
                </tbody>
           </table>
           <div class="py-10">
                <div class={newRentalModalOpen() ? "modal modal-open" : "modal"}>
                    <div class="modal-box">
                        <h3 class="font-bold text-lg">Neues Fahrzeug anlegen</h3>
                        <div class="flex flex-wrap -mx-3 mb-6">
                            <div class="w-full md:w-1/2 px-3 mb-6 md:mb-0 py-3">
                                <div class="dropdown">
                                     <label tabindex="0" class="btn m-1">{ newPerson().hasOwnProperty("id") === true ? newPerson().prename + " " + newPerson().surname : "- bitte wählen -"}</label>
                                     <ul tabindex="0" class="dropdown-content menu p-2 shadow bg-base-100 rounded-box w-52">
                                         { persons() !== undefined ? persons().map(person => <li onClick={ () => setNewPerson(person)}><a>{person.prename + " " + person.surname}</a></li>) : "" }
                                     </ul>
                                </div>
                            </div>
                            <div class="w-full md:w-1/2 px-3 mb-6 md:mb-0 py-3">
                                <div class="dropdown">
                                        <label tabindex="0" class="btn m-1">{ newCar().hasOwnProperty("id") === true ? newCar().name + " | " + newCar().license : "- bitte wählen -"}</label>
                                        <ul tabindex="0" class="dropdown-content menu p-2 shadow bg-base-100 rounded-box w-52">
                                            { cars() !== undefined ? cars().map(car => <li onClick={ () => setNewCar(car)}><a>{car.name + " | " + car.license}</a></li>) : "" }
                                        </ul>
                                </div>
                            </div>
                            <div class="w-full md:w-1/2 px-3  py-3">
                              <label class="block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2" for="grid-new-start">
                                Start
                              </label>
                              <input value={newStart()} onInput = { (e) => setNewStart(e.target.value) }  class="appearance-none block w-full bg-gray-200 text-gray-700 border border-gray-200 rounded py-3 px-4 leading-tight focus:outline-none focus:bg-white focus:border-gray-500" id="grid-new-start" type="text" placeholder="Start (DD.MM.YYYY)"/>                                    
                            </div>
                            <div class="w-full md:w-1/2 px-3  py-3">
                              <label class="block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2" for="grid-new-end">
                                Ende
                              </label>
                              <input value={newEnd()} onInput = { (e) => setNewEnd(e.target.value) }  class="appearance-none block w-full bg-gray-200 text-gray-700 border border-gray-200 rounded py-3 px-4 leading-tight focus:outline-none focus:bg-white focus:border-gray-500" id="grid-new-end" type="text" placeholder="Ende (DD.MM.YYYY)"/>                                                              
                            </div>
                        </div>
                        <div class="modal-action">
                            <label for="modal-new-car" class="btn" onClick={ () => { postRental(); setNewRentalModalOpen(false); } }>Speichern</label>
                            <label for="modal-new-car" class="btn" onClick={ () => setNewRentalModalOpen(false)}>Abbrechen</label>
                        </div>
                    </div>
                    
                </div>
            </div>   
            <div> 
                {/* Modal zum Ändern der Ausleihe */}
                <div class={changeRentalModalOpen() ? "modal modal-open" : "modal"}>
                    <div class="modal-box">
                        <h3 class="font-bold text-lg">Ausleihe ändern</h3>
                        <div class="flex flex-wrap -mx-3 mb-6">
                            <div class="w-full md:w-1/2 px-3 mb-6 md:mb-0 py-3">
                                <div class="dropdown">
                                     <label tabindex="0" class="btn m-1">{ person().hasOwnProperty("id") ? person().prename + " " + person().surname : "- ungültig  -"}</label>
                                     <ul tabindex="0" class="dropdown-content menu p-2 shadow bg-base-100 rounded-box w-52">
                                         { persons() !== undefined ? persons().map(person => <li onClick={ () => setPerson(person)}><a>{person.prename + " " + person.surname + " " + person.age}</a></li>) : "" }
                                     </ul>
                                </div>
                            </div>
                            <div class="w-full md:w-1/2 px-3 mb-6 md:mb-0 py-3">
                                <div class="dropdown">
                                        <label tabindex="0" class="btn m-1">{ car().hasOwnProperty("id") !== "" ? car().name + " | " + car().license : "- ungültig -"}</label>
                                        <ul tabindex="0" class="dropdown-content menu p-2 shadow bg-base-100 rounded-box w-52">
                                            { cars() !== undefined ? cars().map(car => <li onClick={ () => setCar(car)}><a>{car.name + " | " + car.license}</a></li>) : "" }
                                        </ul>
                                </div>
                            </div>
                            <div class="w-full md:w-1/2 px-3  py-3">
                              <label class="block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2" for="grid-new-start">
                                Start
                              </label>
                              <input value={start()} onInput = { (e) => setStart(e.target.value) }  class="appearance-none block w-full bg-gray-200 text-gray-700 border border-gray-200 rounded py-3 px-4 leading-tight focus:outline-none focus:bg-white focus:border-gray-500" id="grid-new-start" type="text" placeholder="Start (DD.MM.YYYY)"/>                                    
                            </div>
                            <div class="w-full md:w-1/2 px-3  py-3">
                              <label class="block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2" for="grid-new-end">
                                Ende
                              </label>
                              <input value={end()} onInput = { (e) => setEnd(e.target.value) }  class="appearance-none block w-full bg-gray-200 text-gray-700 border border-gray-200 rounded py-3 px-4 leading-tight focus:outline-none focus:bg-white focus:border-gray-500" id="grid-new-end" type="text" placeholder="Ende (DD.MM.YYYY)"/>                                                              
                            </div>
                        </div>
                        <div class="modal-action">
                            <label for="my-modal" class="btn" onClick={ () => { putRental(); setChangeRentalModalOpen(false); } }>Speichern</label>
                            <label for="my-modal" class="btn" onClick={ () => setChangeRentalModalOpen(false)}>Abbrechen</label>
                        </div>
                    </div>
                </div>
            </div>
    </div>)  
}
