import type { APIRoute } from 'astro';
import sqlite from 'better-sqlite3';
import path from 'path';
import { v4 as uuidv4 } from 'uuid';

const dbPath = path.resolve("./src/db/", 'rental.db');

export async function get({params, request}) {
  let db = new sqlite(dbPath);
  let personsFromDb = await db.prepare('SELECT * FROM Persons').all();
  db.close();
  return {
    body: JSON.stringify({
      personObject: {
        persons: personsFromDb, 
        success: "ok",
        errorMessage: ""
      }
    })
  };
}
  

export async function post({params, request}) {
  // überprüfen, ob alle Daten vorhanden sind. 
  // Die Daten werden in dem Body übertragen. 
  // Diese Daten im Body lassen sich durch umwandeln 
  // des JSON Strings 
  let body = await request.json();
  if ( body.hasOwnProperty("prename")
    && body.hasOwnProperty("surname")
    && body.hasOwnProperty("age")
    && body.hasOwnProperty("city")) {
      let id = uuidv4();
      let prename = body.prename; 
      let surname = body.surname;
      let age = body.age;
      let city = body.city;
      let db = new sqlite(dbPath);
      let added = db.prepare("INSERT INTO Persons (id, prename, surname, age, city) VALUES (?,?,?,?,?)")
                  .run(id, prename, surname, age, city);
      db.close();
      return {
        body: JSON.stringify({
          success: "ok",
          message: "new person added"
        })
      };
    } else {
      return {
        body: JSON.stringify({
          success: "error",
          message: "attributes missing"
        })
      }
    }
}
  
export async function put({params, request}) {
  // überprüfen, ob alle Daten vorhanden sind. 
  // Die Daten werden in dem Body übertragen. 
  // Diese Daten im Body lassen sich durch umwandeln 
  // des JSON Strings 
  let person = await request.json();
  if ( person.hasOwnProperty("id")
    && person.hasOwnProperty("prename")
    && person.hasOwnProperty("surname")
    && person.hasOwnProperty("age")
    && person.hasOwnProperty("city")) {
      let db = new sqlite(dbPath);
      const updates = db.prepare('UPDATE Persons SET prename = ?, surname = ?, age = ?, city = ? WHERE id = ?')
                     .run(person.prename, person.surname, person.age, person.city, person.id); 
      db.close();
      return {
        body: JSON.stringify({
          success: "ok",
          message: updates.changes + " person updated"
        })
      };
    } else {
      return {
        body: JSON.stringify({
          success: "error",
          message: "attributes missing"
        })
      }
    }
}

export async function del({params, request}) {
  // überprüfen, ob alle Daten vorhanden sind. 
  // Die Daten werden in dem Body übertragen. 
  // Diese Daten im Body lassen sich durch umwandeln 
  // des JSON Strings 
  let body = await request.json();
  if ( body.hasOwnProperty("id")) {
      let db = new sqlite(dbPath);
      const updates = db.prepare('DELETE FROM Persons WHERE id = ?')
                        .run(body.id); 
      db.close();
      return {
        body: JSON.stringify({
          success: "ok",
          message: updates.changes + " person deleted"
        })
      };
    } else {
      return {
        body: JSON.stringify({
          success: "error",
          message: "attributes missing"
        })
      }
    }
}


