import type { APIRoute } from 'astro';
import sqlite from 'better-sqlite3';
import path from 'path';
import { v4 as uuidv4 } from 'uuid';
import dayjs from 'dayjs';

const dbPath = path.resolve("./src/db/", 'rental.db');

export async function get({params, request}) {
  let db = new sqlite(dbPath);
  let rentalsFromDb = await db.prepare('SELECT * FROM Rentals').all();
  db.close(); 
  return {
    body: JSON.stringify({
      rentalsObject: {
        rentals: rentalsFromDb, 
        success: "ok",
        errorMessage: ""
      }
    })
  };
}
  

export async function post({params, request}) {
  let rental = await request.json();
  if ( rental.hasOwnProperty("carId")
    && rental.hasOwnProperty("personId")
    && rental.hasOwnProperty("start")
    && rental.hasOwnProperty("end")) {
      let id = uuidv4();
      let db = new sqlite(dbPath);
      let added = db.prepare("INSERT INTO Rentals (id, carId, personId, start, end, return) VALUES (?,?,?,?,?,?)")
                  .run(id, rental.carId, rental.personId, rental.start, rental.end, 0);
      db.close();
      return {
        body: JSON.stringify({
          success: "ok",
          message: "new rental added"
        })
      };
    } else {
      return {
        body: JSON.stringify({
          success: "error",
          message: "attributes missing"
        })
      }
    }
}
  
export async function put({params, request}) {
  // überprüfen, ob alle Daten vorhanden sind. 
  // Die Daten werden in dem Body übertragen. 
  // Diese Daten im Body lassen sich durch umwandeln 
  // des JSON Strings 
  let rental = await request.json();
  if ( rental.hasOwnProperty("id")
      && rental.hasOwnProperty("carId")
      && rental.hasOwnProperty("personId")
      && rental.hasOwnProperty("start")
      && rental.hasOwnProperty("end")) {

      let db = new sqlite(dbPath);
      const updates = db.prepare('UPDATE Rentals SET carId = ?, personId = ?, start = ?, end = ?, return = 0 WHERE id = ?')
                     .run(rental.carId, rental.personId, rental.start, rental.end, rental.id); 
      db.close();
      return {
        body: JSON.stringify({
          success: "ok",
          message: updates.changes + " rental updated"
        })
      };
    } else {
      return {
        body: JSON.stringify({
          success: "error",
          message: "attributes missing"
        })
      }
    }
}

export async function del({params, request}) {
  // überprüfen, ob alle Daten vorhanden sind. 
  // Die Daten werden in dem Body übertragen. 
  // Diese Daten im Body lassen sich durch umwandeln 
  // des JSON Strings 
  let body = await request.json();
  if ( body.hasOwnProperty("id")) {
      let db = new sqlite(dbPath);
      const updates = db.prepare('DELETE FROM Rentals WHERE id = ?')
                        .run(body.id); 
      db.close();
      return {
        body: JSON.stringify({
          success: "ok",
          message: updates.changes + " rentals deleted"
        })
      };
    } else {
      return {
        body: JSON.stringify({
          success: "error",
          message: "attributes missing"
        })
      }
    }
}


